/*
 * AlibavaMonitor.cc
 *
 *  Created on: Jul 20, 2012
 *      Author: lacasta
 */

#include <iomanip>
#include <sstream>
#include "AlibavaMonitor.h"
#include "AlibavaGUI.h"

AlibavaMonitor::AlibavaMonitor()
    : _time_elapsed("0 0:00:00"), _rate("0 Hz"),
      _ntrigger(0), _nexpected(0), _naccepted(0), _ntrig0(0.0), _tim0(0.0), _data(0)
{
    // TODO Auto-generated constructor stub

}

double AlibavaMonitor::getFraction() const
{
    return _fraction;
}

int AlibavaMonitor::getNtrigger() const
{
    return _ntrigger;
}

const std::string& AlibavaMonitor::getOutFile() const
{
    return _out_file;
}

const std::string& AlibavaMonitor::getRate() const
{
    return _rate;
}

double AlibavaMonitor::getRateAsNumber() const
{
    return _ratef;
}

void AlibavaMonitor::setFraction(double fraction)
{
    _fraction = fraction;
}

void AlibavaMonitor::setNtrigger(int ntrigger)
{
    _ntrigger = ntrigger;
}

void AlibavaMonitor::setOutFile(const std::string& outFile)
{
    _out_file = outFile;
}

void AlibavaMonitor::setRate(const std::string& rate)
{
    _rate = rate;
}

void AlibavaMonitor::setRate(double val)
{
    _ratef = val;
}

const std::string& AlibavaMonitor::getRunType() const
{
    return _run_type;
}

int AlibavaMonitor::getNexpected() const
{
    return _nexpected;
}

double AlibavaMonitor::getValue() const
{
    return _value;
}

void AlibavaMonitor::setValue(double value)
{
    _value = value;
}

void AlibavaMonitor::setNexpected(int nexpected)
{
    _nexpected = nexpected;
}

void AlibavaMonitor::setRunType(const std::string& runType)
{
    _run_type = runType;
}

void AlibavaMonitor::setTimeElapsed(const std::string& timeElapsed)
{
    _time_elapsed = timeElapsed;
}

void AlibavaMonitor::setData(void* data)
{
    _data = data;
}

const std::string& AlibavaMonitor::getTimeElapsed() const
{
    return _time_elapsed;
}

AlibavaMonitor::~AlibavaMonitor()
{
    // TODO Auto-generated destructor stub
}

void AlibavaMonitor::getHistogram(const std::string &whst, size_t &size, char **buffer)
{
    AlibavaGUI *gui = (AlibavaGUI *)_data;
    gui->getHistogram(whst, size, buffer);
}


int AlibavaMonitor::getRunTypeVal() const
{
    return _run_type_val;
}
void AlibavaMonitor::setRunTypeVal( int v )
{
    _run_type_val = v;
    switch (_run_type_val )
    {
        case 1:
            _run_type = "Calibration";
            break;
        case 2:
            _run_type = "LaserSync";
            break;
        case 3:
            _run_type = "Laser";
            break;
        case 4:
            _run_type = "RadSource";
            break;
        case 5:
            _run_type = "Pedestal";
            break;
        default:
            _run_type = "Unknown";
            break;
    }
}

double AlibavaMonitor::getNtrig0() const
{
    return _ntrig0;
}

void AlibavaMonitor::setNtrig0(double ntrig0)
{
    _ntrig0 = ntrig0;
}

double AlibavaMonitor::getTim0() const
{
    return _tim0;
}

int AlibavaMonitor::getNaccepted() const
{
    return _naccepted;
}

void AlibavaMonitor::setNaccepted(int naccepted)
{
    _naccepted = naccepted;
}

void AlibavaMonitor::setTim0(double tim0)
{
    _tim0 = tim0;
}

double AlibavaMonitor::getEfficiency() const
{
    double eff = getNtrigger() > 0.0
            ? 100.0*getNaccepted()/getNtrigger()
            : 0.0;
    return eff;
}
std::string AlibavaMonitor::getEfficiencyAsString(int set_prec) const
{
    std::ostringstream ostr;
    double eff = getEfficiency();
    ostr << std::setiosflags(std::ios::left | std::ios::fixed)
         << std::setprecision(set_prec)
         << eff;
    return ostr.str();
}


void AlibavaMonitor::reset_counters()
{
    _ntrig0 = 0;
    _tim0 = 0;
    _naccepted = 0;
    _ntrigger = 0;
}
