#include <iostream>
#include <iomanip>
#include "BeetleConfig.h"

static const char *var_name[] =
{
    "Itp","Ipre","Isha","Ibuf","Vfp","Vfs",
    "Ipipe","Vd","Vdcl","Ivoltbuf","Isf","Icurrbuf","Latency",
    "ROCtrl","RclkDiv","CompCtrl",
    0
};
static const unsigned short var_value[] =
{
    0x0000, 0x014c, 0x020A, 0x030a, 0x0400, 0x0500,
    0x0a0d, 0x0b82, 0x0c69, 0x0d14, 0x0e1a, 0x0f66, 0x1080,
    0x111a, 0x1200, 0x1309,
    0x0700, 0x0800
};


BeetleConfig::BeetleConfig() : _mask(3)
{
    int i;
    set_group("beetle");
    _comparator.set_group("comparator");
    for (i=0; var_name[i]; i++)
        (*this)[var_name[i]] = var_value[i];

}


void BeetleConfig::cpy(const BeetleConfig &b)
{
    const_iterator ip;
    clear();
    for (ip=b.begin(); ip!=b.end(); ++ip)
    {
        (*this)[ip->first] = ip->second;
    }
    _mask = b._mask;
    _comparator = b._comparator;
    set_group(b.get_group() );
}

void BeetleConfig::load_data()
{

    if (exists())
    {
        int i;
        for (i=0; var_name[i]; i++)
        {
            (*this)[var_name[i]] = (var_value[i] & 0xff00) | (0xff &get_int(var_name[i]));
        }
        _mask = get_int("mask", 3);
    }
    if ( _comparator.key_file() != key_file() )
        _comparator.set_file(key_file());
    else
        _comparator.load_data();
}
void BeetleConfig::save_data()
{
    iterator ip;
    for (ip=begin(); ip!=end(); ++ip)
    {
        set_int(ip->first.c_str(), (0xff & ip->second));
    }
    set_int("mask", _mask);
    _comparator.save_data();
}

ComparatorConfig &BeetleConfig::get_comparator_settings()
{
    unsigned short CompCtrl = (*this)["CompCtrl"] & 0xff;

    _comparator.setEnable( (CompCtrl & 9)==0 );
    _comparator.setPolarity( (CompCtrl & 2)/2 );
    _comparator.setMode( (CompCtrl & 16) ? ComparatorConfig::pulseMode : ComparatorConfig::trackMode );

    return _comparator;
}

void BeetleConfig::set_comparator_settings(const ComparatorConfig &comparator)
{
    _comparator = comparator;
    unsigned short ctrl = 0;
    if ( !_comparator.isEnabled() )
    {
        ctrl = 9;
    }
    if ( _comparator.getPolarity() )
        ctrl |= 2;

    if ( _comparator.getMode() == ComparatorConfig::pulseMode )
        ctrl |= 16;


    unsigned short CompCtrl = (*this)["CompCtrl"] & 0xff00;
    (*this)["CompCtrl"] = CompCtrl | ctrl;
}
