/*
 * DAQmanager.h
 *
 *  Created on: 29 Jan 2011
 *      Author: lacasta
 */

#ifndef DAQMANAGER_H_
#define DAQMANAGER_H_

#include <string>
#include <RunManager.h>
#include <Alibava.h>
#include <DataFile.h>
#include <DataMonitor.h>
#include <KeyFile.h>
#include <glib.h>

class DAQmanager : public DataReceiver
{
    private:
        int nevts;
        int received;
        int expected;
        Alibava *board;
        DataFile *ofile;
        DataMonitor *monitor;
        RunManager::RunType run_type;
        RunManager *rm;
        GMutex *mtx;

        bool _starting;
        bool _running;
        bool _stop_running;

        // Config file
        KeyFile *config_file;

        void running(bool x);

        /** TODO:
         * Implement DAtaMOnitor in RunManager instead of AlibabaGUI
         */

    public:
        DAQmanager(int nn, Alibava *B, RunManager::RunType tp = RunManager::Unknown);
        virtual ~DAQmanager();

        void set_expected(int x) { expected = nevts = x; }
        void set_received(int x) { received = x; }
        int get_expected() const { return expected; }
        int get_received() const { return received; }
        double get_fraction_received() const { return expected > 0.0 ? double(received)/double(expected) : 0.0; }

        void set_monitor(DataMonitor *m) { monitor = m; }
        DataMonitor *get_monitor() { return monitor; }

        void set_data_file(const std::string &fnam);
        void set_run_type( RunManager::RunType tp);
        RunManager::RunType get_run_type() { return run_type; }
        RunManager *get_run_manager() { return rm; }

        DataFile *out_file() { return ofile; }

        virtual int open(const char *fnam);
        virtual int reset();
        virtual int prepare_for_run();
        virtual int acquire();
        virtual int stop_run();
        virtual void device_opened() {}
        virtual void receive (const EventData *);

        virtual void process_event(const EventData *);

        Alibava *get_board() { return board; }


        // tells if the acquisition is getting ready to start
        bool starting() const;

        // tells if the acquisition is running
        bool running() const;

        // returns true if run has to stop
        bool stop() const;

        // sets the stop condition
        void stop(bool x);


        // Configuration
        void save_config(const char *);
        void load_config(const char *);

};

#endif /* DAQMANAGER_H_ */
