#include <cstring>
#include "DataBuffer.h"

std::allocator<char> DataBlock::data_allocator;
std::allocator<DataBlock> DataBlock::buffer_allocator;

DataBlock::DataBlock(int s, const char *d) :
   _size(s), _data(0)
{
   _data = data_allocator.allocate(_size);
   if (d)
   {
      memcpy(_data, d, _size);
   }
   else
   {
      memset(_data, 0, _size);
   }
}

DataBlock::~DataBlock()
{
   data_allocator.deallocate(_data, _size);
}

void DataBlock::cpy(const DataBlock &d)
{
   data_allocator.deallocate(_data, _size);
   _data = 0;
   _size = d._size;
   if (d._data)
   {
      _data = data_allocator.allocate(_size);
      memcpy(_data, d._data, _size);
   }
}

DataBuffer::DataBuffer(int sz) :
   _mxsize(sz)
{
}

DataBuffer::~DataBuffer()
{
   while (!empty())
   {
      DataBlock *data = front();
      pop();
      delete data;
   }
}

void DataBuffer::push_block(DataBlock *b)
{
   if ( !full() )
   {
      push(b);
   }
   else
      delete b;
}

DataBlock *DataBuffer::get_block()
{
   if (empty() )
      return 0;

   DataBlock *b = front();
   pop();

   return b;
}

void DataBuffer::flush()
{
    DataBlock *blk;
    while ( (blk=get_block()))
    {
        delete blk;
    }
}
