#include <iostream>
#include <iomanip>
#include <algorithm>
#include <string>
#include <sstream>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <cerrno>
#include <sys/types.h>
#include <unistd.h>
#include "string_utils.h"
#include "KeyFile.h"

/**
 * Set of utils
 *
 */
namespace
{
//    inline bool isws(char c, char const * const wstr=" \t\n")
//    {
//        return (strchr(wstr, c) != NULL);
//    }
//
//    inline std::string trim_right(const std::string &s)
//    {
//        std::string b=" \t\n";
//        std::string str = s;
//        return str.erase(str.find_last_not_of(b) +1);
//    }
//    inline std::string trim_left(const std::string &s)
//    {
//        std::string b=" \t\n";
//        std::string str = s;
//        return str.erase( 0, str.find_first_not_of(b) );
//    }
//    inline std::string trim_str(const std::string &s)
//    {
//        std::string str = s;
//        return trim_left(trim_right(str) );
//    }
//
//    struct ToLower
//    {
//            char operator()(char c) const
//            {
//                return std::tolower(c);
//            }
//    };
//    inline std::string lower_string(std::string s)
//    {
//        std::transform(s.begin(), s.end(), s.begin(), ToLower() );
//        return s;
//    }
    bool parse_as_bool(const std::string &str) throw (KeyFile::Exception)
    {
        std::string s = lower_string(str);
        if (s=="yes" || s=="true" || s=="1")
            return true;

        if (s=="no" || s=="false" || s=="0")
            return false;

        std::ostringstream ostr;
        ostr << "Error: value " << str << " cannot be interpreted as a boolean";
        throw KeyFile::Exception( ostr.str());
    }

    double parse_as_double(const std::string &str) throw (KeyFile::Exception)
    {
        char *end;
        const char *value=str.c_str();
        double rc=0.;

        rc = strtod(value, &end);
        if ( *end!='\0' || end==value)
        {
            std::ostringstream ostr;
            ostr << "Error: value " << str << " cannot be interpreted as double";
            throw KeyFile::Exception( ostr.str() );
        }
        return rc;
    }

    int parse_as_int(const std::string &str) throw (KeyFile::Exception)
    {
        char *end;
        const char *value=str.c_str();
        long long_value;
        int int_value;

        errno=0;
        long_value = strtol(value, &end, 10);
        if ( *end!='\0' || end==value)
        {
            std::ostringstream ostr;
            ostr << "Error: value " << str << " cannot be interpreted as integer";
            throw KeyFile::Exception( ostr.str() );
        }
        int_value = long_value;
        if (int_value!=long_value || errno == ERANGE)
        {
            std::ostringstream ostr;
            ostr << "Error: integer " << str << " out of range";
            throw KeyFile::Exception( ostr.str() );
        }
        return int_value;
    }
}

// check if a current group exists
bool KeyFile::has_group(const std::string&s) const
{
    return _groups.find(s) == _groups.end() ? false : true;
}

// check if a given group has a given key
bool KeyFile::has_key(const std::string &g, const std::string &k) const
{
    GroupList::const_iterator gp = _groups.find(g);
    if (gp!=_groups.end())
    {
        ItemList::const_iterator kp = gp->second.find(k);
        return kp==gp->second.end() ? false : true;
    }
    return false;
}

KeyFile::GroupList::iterator KeyFile::add_group(const std::string &g)
{
    GroupList::iterator gp = _groups.find(g);
    if (gp==_groups.end())
    {
        std::pair<GroupList::iterator, bool> rc =_groups.insert(GroupList::value_type(g, ItemList()));

        return rc.first;
    }
    return gp;
}

void KeyFile::add_key(const std::string &g, const std::string &k, const std::string &val)
{
    GroupList::iterator gp = add_group(g);
    if (has_key(g,k))
        gp->second[k] = val;
    else
        gp->second.insert(ItemList::value_type(k, val) );
}

void KeyFile::del_key(const std::string &g, const std::string &k)
{
    GroupList::iterator gp = _groups.find(g);
    if (gp!=_groups.end())
    {
        gp->second.erase(k);
    }
}

void KeyFile::del_group(const std::string &g)
{
    GroupList::iterator gp = _groups.find(g);
    if (gp!=_groups.end())
    {
        gp->second.clear();
        _groups.erase(gp);
    }
}


KeyFile::Exception::Exception()
{
}

KeyFile::Exception::Exception(const KeyFile::Exception &e)
{
    _why = e._why;
}

KeyFile::Exception::~Exception() throw()
{
}

static char buffer[2048];

const char* KeyFile::Exception::what() const throw()
{
    std::string rc;
    if (_why.empty() )
        rc = "KeyFile::Exception";
    else
        rc = _why;

    sprintf(buffer,"process (%d) %s", getpid(), rc.c_str());
    return buffer;
}

KeyFile::KeyFile()
{

}

KeyFile::~KeyFile()
{
    reset();
}

void KeyFile::reset()
{
    GroupList::iterator gp;
    for (gp=_groups.begin(); gp!=_groups.end(); ++gp)
    {
        gp->second.clear();
    }
    _groups.clear();
}

std::string KeyFile::value(const std::string &g, const std::string &k) const throw(Exception)
{
    std::string rc;
    GroupList::const_iterator gp = _groups.find(g);
    if (gp!=_groups.end())
    {
        ItemList::const_iterator kp = gp->second.find(k);
        if (kp == gp->second.end() )
        {
            std::ostringstream ostr;
            ostr << "Key "<< k << " does not exist in group "<< g;
            throw Exception(ostr.str().c_str());
        }
        else
            return kp->second;
    }
    else
    {
        std::ostringstream ostr;
        ostr << "Group "<< g << " does not exist";
        throw Exception(ostr.str().c_str());
    }
}

int KeyFile::load(std::istream &is)
{
    std::string str;
    std::string group;
    int iline=0;

    if (!is)
    {
        std::cout << "KeyFile::load: Problem with input file" << std::endl;
        return -1;
    }

    while (is)
    {
        getline(is, str);
        str = trim_right(str);
        if (str.empty() || str[0]=='#')
            continue;

        // At this stage we should have either a group or a key/value pair
        std::string::size_type bleft, bright;
        if ( (bleft = str.find('[')) == str.npos)
        {
            // we should have a pair or an error
            std::string::size_type offs = str.find('=');
            if (offs == str.npos)
            {
                // problem
                std::cout << "Error in line "<< iline << std::endl;
                break;
            }
            else
            {
                int comment = str.find('#');
                std::string key = trim_str(str.substr(0, offs) );
                std::string val = trim_str(str.substr(offs+1, comment) );
                add_key(group, key, val);
            }
        }
        else
        {
            // A group
            bright = str.find(']');
            if (bright == str.npos)
            {
                // problem
                std::cout << "Error in line "<< iline << std::endl;
                break;
            }
            group = trim_str(str.substr(bleft+1, bright-1));
            add_group(group);
        }
        iline++;
    }
    return 0;
}
int KeyFile::save(std::ostream &os) const
{
    GroupList::const_iterator gp;
    ItemList::const_iterator kp;

    for (gp=_groups.begin(); gp!=_groups.end(); ++gp)
    {
        os << '['<< gp->first<< ']'<< std::endl;
        for (kp=gp->second.begin(); kp!=gp->second.end(); ++kp)
        {
            os << kp->first<< '='<< kp->second<< std::endl;
        }
        os << std::endl;
    }
    return 0;
}

std::ostream &operator<<(std::ostream &os, const KeyFile &kf)
{
    kf.save(os);
    return os;
}

std::istream &operator>>(std::istream &is, KeyFile &kf)
{
    kf.load(is);
    return is;
}

bool KeyFile::get_as_bool(const std::string &g, const std::string &k) const throw(KeyFile::Exception)
{
    std::string val;
    try
    {
        val = value(g, k);
        return parse_as_bool( val );
    }
    catch (Exception e)
    {
        throw;
    }
}

void KeyFile::get_as_bool_list(const std::string &g, const std::string &k, std::vector<bool> &v) throw(KeyFile::Exception)
{
    std::string val;
    try
    {
        val = value(g, k);
    }
    catch (Exception e)
    {
        std::ostringstream ostr;
        ostr << "The key " << g << "::" << k << " does not exist";
        throw Exception(ostr.str());
    }
    std::string::size_type S=val.size();
    std::string::size_type i=0;
    while (i<S)
    {
        std::string::size_type j = val.find(';', i);
        try
        {
            bool rc = parse_as_bool(val.substr(i,j-i));
            v.push_back(rc);
        }
        catch ( Exception e )
        {
            throw;
        }
        if (j==val.npos)
            break;
        i = j+1;
    };
}

void KeyFile::set_as_bool_list(const std::string &g, const std::string &k, const std::vector<bool> &v)
{
    if (v.empty())
        return;

    std::ostringstream ostr;
    std::vector<bool>::const_iterator ip;
    std::vector<bool>::size_type S = v.size()-1;
    for (ip=v.begin(); ip!=v.end(); ++ip, S--)
    {
        ostr << (*ip ? "true" : "false");
        if (S)
            ostr << ';';
    }
    add_key(g, k, ostr.str());
}

void KeyFile::set_as_bool(const std::string &g, const std::string &k, bool x)
{
    add_key(g, k, x ? "true" : "false");
}

double KeyFile::get_as_double(const std::string &g, const std::string &k) const throw(KeyFile::Exception)
{
    std::string val;
    try
    {
        val = value(g,k);
        return parse_as_double(val);
    }
    catch (Exception e)
    {
        throw;
    }
}

void KeyFile::get_as_double_list(const std::string &g, const std::string &k, std::vector<double> &v) throw(KeyFile::Exception)
{
    std::string val;
    try
    {
        val = value(g, k);
    }
    catch (Exception e)
    {
        std::ostringstream ostr;
        ostr << "The key " << g << "::" << k << " does not exist";
        throw Exception(ostr.str());
    }
    std::string::size_type S=val.size();
    std::string::size_type i=0;
    while (i<S)
    {
        std::string::size_type j = val.find(';', i);
        try
        {
            double rc = parse_as_double(val.substr(i,j-i));
            v.push_back(rc);
        }
        catch ( Exception e )
        {
            throw;
        }
        if (j==val.npos)
            break;
        i = j+1;
    };
}

void KeyFile::set_as_double(const std::string &g, const std::string &k, double val)
{
    std::stringstream ostr;
    ostr << std::setiosflags(std::ios::scientific) << std::setprecision(10) << val;
    add_key(g, k, ostr.str());
}
void KeyFile::set_as_double_list(const std::string &g, const std::string &k, const std::vector<double> &v)
{
    if (v.empty())
        return;

    std::ostringstream ostr;
    std::vector<double>::const_iterator ip;
    std::vector<double>::size_type S = v.size()-1;
    for (ip=v.begin(); ip!=v.end(); ++ip, S--)
    {
        ostr << std::setiosflags(std::ios::scientific) << std::setprecision(10) << *ip;
        if (S)
            ostr << ';';
    }
    add_key(g, k, ostr.str());
}

int KeyFile::get_as_int(const std::string &g, const std::string &k) const throw(KeyFile::Exception)
{
    std::string val;
    try
    {
        val = value(g,k);
        return parse_as_int(val);
    }
    catch (Exception e)
    {
        throw;
    }
}
void KeyFile::get_as_int_list(const std::string &g, const std::string &k, std::vector<int> &v) throw(KeyFile::Exception)
{
    std::string val;
    try
    {
        val = value(g, k);
    }
    catch (Exception e)
    {
        std::ostringstream ostr;
        ostr << "The key " << g << "::" << k << " does not exist";
        throw Exception(ostr.str());
    }
    std::string::size_type S=val.size();
    std::string::size_type i=0;
    while (i<S)
    {
        std::string::size_type j = val.find(';', i);
        try
        {
            int rc = parse_as_int(val.substr(i,j-i));
            v.push_back(rc);
        }
        catch ( Exception e )
        {
            throw;
        }
        if (j==val.npos)
            break;
        i = j+1;
    };

}

void KeyFile::set_as_int(const std::string &g, const std::string &k, int val)
{
    std::ostringstream ostr;
    ostr << val;
    add_key(g, k, ostr.str());
}

void KeyFile::set_as_int_list(const std::string &g, const std::string &k, const std::vector<int> &v)
{
    if (v.empty())
        return;

    std::ostringstream ostr;
    std::vector<int>::const_iterator ip;
    std::vector<int>::size_type S = v.size()-1;
    for (ip=v.begin(); ip!=v.end(); ++ip, S--)
    {
        ostr << *ip;
        if (S)
            ostr << ';';
    }
    add_key(g, k, ostr.str());
}
